"""Callback handlers used in the app."""
from typing import Any
from rag_doc_search.src.models.chat_response import ChatResponse

from langchain.callbacks.base import AsyncCallbackHandler


class StreamingLLMCallbackHandler(AsyncCallbackHandler):
    """Callback handler for streaming LLM responses."""

    def __init__(self, websocket):
        self.websocket = websocket

    async def on_llm_new_token(self, token: str, **kwargs: Any) -> None:
        """
        Asynchronously handles the event triggered when a new token is encountered in the language model and sends the token in websocket event.

        Parameters:
        - `token`: A string representing the new token encountered.
        - `**kwargs`: Additional keyword arguments that may be provided (optional).

        Returns:
        None
        """
        resp = ChatResponse(sender="bot", message=token, type="stream")
        await self.websocket.send_json(resp.dict())



from enum import Enum


class VectorStoreProvider(Enum):
    """
    Enum representing the providers available for a vector stores.

    Attributes:
    - `PGVector`: Refers to the PGVector provider.
    - `FAISS`: Refers to the FAISS provider.
    """
    PGVector = "PGVector"
    FAISS = "FAISS"


class AIProvider(Enum):
    """
    Enum representing providers available for the AI model API.

    Attributes:
    - `OPENAI`: Refers to the OpenAI provider.
    - `AZURE_OPENAI`: Refers to the Azure OpenAI provider.
    - `BEDROCK`: Refers to the Bedrock provider.
    """
    OPENAI = "OPENAI"
    AZURE_OPENAI = "AZURE_OPENAI"
    BEDROCK = "BEDROCK"



from pydantic import BaseModel, validator

"""Schemas for the chat app."""


class ChatResponse(BaseModel):
    """Chat response schema."""

    sender: str
    message: str
    type: str

    @validator("sender")
    def sender_must_be_bot_or_you(cls, v):
        if v not in ["bot", "human"]:
            raise ValueError("sender must be bot or human")
        return v

    @validator("type")
    def validate_message_type(cls, v):
        if v not in ["start", "stream", "end", "error", "info"]:
            raise ValueError("type must be start, stream or end")
        return v




from .src.enums.provider import AIProvider
from .src.bot_models.chatbot_model import ChatBotModel
from .utils.config import Config

def config_init(config_json:dict)-> Config:
    """
    Initializes the configuration based on the provided JSON config.

    Parameters:
    - config_json (dict): A dictionary containing configuration parameters.

    Returns: (Config) -> Initialized Config instance

    None
    """
    global config
    if(config_json is None):
        raise ValueError("Config Json can not be None or Null")
    config = Config(config_json)
    
    return config

def get_bot_instance(ai_provider : AIProvider) -> ChatBotModel: 
    """
    Returns an instance of the ChatBotModel based on the specified AI provider.

    Parameters:
    - ai_provider (AIProvider): An enum specifying the AI provider for the ChatBotModel.

    Returns:
    ChatBotModel: An instance of the ChatBotModel corresponding to the specified AI provider.

    Raises:
    ValueError: If the specified AI provider is not supported.
    """
    match ai_provider:
        case AIProvider.BEDROCK:
            from rag_doc_search.src.bot_models.bedrock_chatbot_model import BedrockChatBot
            return BedrockChatBot()
        case AIProvider.OPENAI:
            from rag_doc_search.src.bot_models.openai_chatbot_model import OpenAIChatBot
            return OpenAIChatBot()
        case AIProvider.AZURE_OPENAI:
            from rag_doc_search.src.bot_models.azure_chatbot_model import AzureChatBot
            return AzureChatBot()
        case _:
            raise ValueError("cannot initiate llm base model for given ai provider")
        
# Export the init function
__all__ = ["init"]



from langchain.prompts import PromptTemplate
from langchain.chains import ConversationalRetrievalChain
from langchain.memory import ConversationBufferWindowMemory
from langchain.callbacks.manager import AsyncCallbackManager
from langchain.callbacks.tracers import LangChainTracer
from langchain.schema.vectorstore import VectorStore
from langchain.schema.embeddings import Embeddings
from langchain.chains import RetrievalQA
from langchain.schema.language_model import BaseLanguageModel

from rag_doc_search.src.prompt_templates.default_prompt_templates import (
    DEFAULT_PROMPT_TEMPLATE,
    DEFAULT_CHAT_HISTORY_PROMPT,
)
from rag_doc_search.utils.miscellaneous import get_chat_history
from rag_doc_search.utils.miscellaneous import get_logger


class ChatBotModel:
    """
    A class representing the Base ChatBot.

    This class serves as an implementation of a base chatbot for diff LLM's.
    """

    def __init__(
        self, embeddings: Embeddings, vector_store: VectorStore, retriever_args: dict
    ):
        self.prompt_template = DEFAULT_PROMPT_TEMPLATE
        self.embeddings = (embeddings,)
        self.vector_store = vector_store
        self.logger = get_logger()
        self.retriever_args = retriever_args
        self.logger.info(
            f"search_type: {self.retriever_args.get('search_type')} \n search_args: {self.retriever_args.get('search_args')}"
        )

    def create_stream_manager(self, stream_handler, tracing) -> AsyncCallbackManager:
        """
        Creates and returns an instance of AsyncCallbackManager for handling asynchronous callbacks.

        Parameters:
        - `stream_handler`: The handler for managing the stream.
        - `tracing`: A boolean indicating whether tracing is enabled.

        Returns:
        An instance of AsyncCallbackManager.
        """
        manager = AsyncCallbackManager([])
        stream_manager = AsyncCallbackManager([stream_handler])
        if tracing:
            tracer = LangChainTracer()
            tracer.load_default_session()
            manager.add_handler(tracer)
            stream_manager.add_handler(tracer)

        return stream_manager

    def create_qa_chain(self, cl_llm: BaseLanguageModel) -> RetrievalQA:
        """
        Creates and returns an instance of RetrievalQA for question-answering using a provided language model.

        Parameters:
        - `cl_llm`: An instance of LanguageModel such as OpenAI or Bedrock Model.

        Returns:
        An instance of RetrievalQA.
        """
        PROMPT = PromptTemplate(
            template=self.prompt_template, input_variables=["context", "question"]
        )

        qa = RetrievalQA.from_chain_type(
            llm=cl_llm,
            chain_type="stuff",
            retriever=self.vector_store.as_retriever(
                search_type=self.retriever_args.get("search_type"),
                search_args=self.retriever_args.get("search_args"),
            ),
            return_source_documents=True,
            chain_type_kwargs={"prompt": PROMPT},
        )
        return qa

    def create_conversational_qa_chain(
        self, cl_llm: BaseLanguageModel
    ) -> ConversationalRetrievalChain:
        """
        Creates and returns an instance of ConversationalRetrievalChain for handling conversational question-answering
        using a provided language model.

        Parameters:
        - `cl_llm`: An instance of LanguageModel such as OpenAI or Bedrock Model.

        Returns:
        An instance of ConversationalRetrievalChain.
        """
        memory_chain = ConversationBufferWindowMemory(
            memory_key="chat_history",
            ai_prefix="Assistant",
            return_messages=True,
            k=0,
            output_key="answer",
        )

        # the condense prompt for Claude
        condense_prompt = PromptTemplate.from_template(DEFAULT_CHAT_HISTORY_PROMPT)

        qa = ConversationalRetrievalChain.from_llm(
            llm=cl_llm,
            retriever=self.vector_store.as_retriever(
                search_type=self.retriever_args.get("search_type"),
                search_args=self.retriever_args.get("search_args"),
            ),
            # return_source_documents=True,
            memory=memory_chain,
            get_chat_history=get_chat_history,
            condense_question_prompt=condense_prompt,
            chain_type="stuff",  # 'refine',
        )

        # the LLMChain prompt to get the answer. the ConversationalRetrievalChange does not expose this parameter in the constructor
        qa.combine_docs_chain.llm_chain.prompt = PromptTemplate.from_template(
            self.prompt_template
        )
        return qa



from enum import Enum


class RetrieverSearchType(Enum):
    """
    Enum representing the search types available for a vectors retriever.

    Attributes:
    - `mmr`: Refers to the search type 'Maximum marginal relevance retrieval'.
    - `similarity`: Refers to the search type 'Similarity search retrieval'.
    - `similarity_score_threshold`: Refers to the search type 'Similarity score threshold retrieval'.
    """
    mmr = "mmr"
    similarity = "similarity"
    similarity_score_threshold = "similarity_score_threshold"



import json
import logging
from langchain.schema import BaseMessage


def get_logger() -> logging.Logger:
    """
        provides logger instance

    Returns:
        logging.Logger: logger for application
    """
    logger = logging.getLogger(__name__)
    if not logger.handlers:
        logger.propagate = 0
        logger.setLevel(logging.DEBUG)
        log_formatter = logging.Formatter(
            "%(levelname)s.%(module)s.%(funcName)s Line:%(lineno)d: %(asctime)s %(message)s"
        )
        stream_handler = logging.StreamHandler()
        stream_handler.setFormatter(log_formatter)
        logger.addHandler(stream_handler)
    return logger


def get_chat_history(chat_history):
    """
    Converts the given chat history into a formatted string representation.

    The chat history may contain messages from both humans and the AI. The function supports
    different formats for representing each dialogue turn.

    Parameters:
    - `chat_history`: A list representing the chat history. Each element can be a `BaseMessage`
      instance or a tuple containing a human message and an AI response.

    Returns:
    A formatted string representation of the chat history.
    """

    _ROLE_MAP = {"human": "\n\nHuman: ", "ai": "\n\nAssistant: "}
    buffer = ""
    for dialogue_turn in chat_history:
        if isinstance(dialogue_turn, BaseMessage):
            role_prefix = _ROLE_MAP.get(dialogue_turn.type, f"{dialogue_turn.type}: ")
            buffer += f"\n{role_prefix}{dialogue_turn.content}"
        elif isinstance(dialogue_turn, tuple):
            human = "\n\nHuman: " + dialogue_turn[0]
            ai = "\n\nAssistant: " + dialogue_turn[1]
            buffer += "\n" + "\n".join([human, ai])
        else:
            raise ValueError(
                f"Unsupported chat history format: {type(dialogue_turn)}."
                f" Full chat history: {chat_history} "
            )
    return buffer


def json_to_dict(path: str) -> dict:
    """
    Convert JSON data from the specified path to a Python dictionary.

    Args:
        path (str): The file path to the JSON file.

    Returns:
        dict: A dictionary containing the data parsed from the JSON file.
    """
    with open(path, "r") as json_file:
        parsed_json = json.load(json_file)
    return parsed_json



from langchain_openai import AzureOpenAIEmbeddings
from langchain.llms.openai import AzureOpenAI
from langchain_openai import AzureChatOpenAI
from langchain.schema.language_model import BaseLanguageModel
from langchain.chains import RetrievalQA
from langchain.chains import ConversationalRetrievalChain

from rag_doc_search.src.bot_models.chatbot_model import ChatBotModel
from rag_doc_search import config

import os


class AzureChatBot(ChatBotModel):
    """
    A class representing the Azure OpenAI ChatBot.

    This class serves as an implementation of a chatbot using the OpenAI.
    """

    def __init__(self):
        self.config = config
        self.embeddings = AzureOpenAIEmbeddings(azure_deployment=self.config.embeddings_model,api_version=os.environ.get("AZURE_OPENAI_API_VERSION"))
        self.vector_store = self.config.get_vector_store(embeddings=self.embeddings)
        super().__init__(
            embeddings=self.embeddings,
            vector_store=self.vector_store,
            retriever_args=self.config.get_retriever_args(),
        )

    def create_qa_instance(self) -> RetrievalQA:
        """
        Creates and returns an instance of RetrivalQA using OpenAI Language Model.

        Returns:
        An instance of RetrivalQA.
        """
        cl_llm: BaseLanguageModel = AzureChatOpenAI(
            azure_deployment = self.config.llm,
            model_name=self.config.llm,
            temperature=self.config.llm_temperature,
            max_tokens=self.config.llm_max_output_tokens,
            openai_api_key=os.environ.get("AZURE_OPENAI_API_KEY"),
            openai_api_version=os.environ.get("AZURE_OPENAI_API_VERSION")
        )
        qa = self.create_qa_chain(cl_llm)
        return qa

    def create_conversational_qa_instance(
        self, stream_handler, tracing: bool = False
    ) -> ConversationalRetrievalChain:
        """
        Creates and returns an instance of ConversationalRetrievalChain for conversational question-answering using OpenAI Language Model.

        Parameters:
        - `stream_handler`: An instance of StreamingLLMCallbackHandler used for handling streaming callbacks.
        - `tracing`: A boolean indicating whether tracing is enabled. Default is False.

        Returns:
        An instance of ConversationalRetrievalChain for conversational question-answering.
        """
        stream_manager = self.create_stream_manager(stream_handler, tracing)
        cl_llm: BaseLanguageModel = AzureChatOpenAI(
            azure_deployment=self.config.llm,
            model_name=self.config.llm,
            temperature=self.config.llm_temperature,
            max_tokens=self.config.llm_max_output_tokens,
            streaming=True,
            callback_manager=stream_manager,
            openai_api_key=os.environ.get("AZURE_OPENAI_API_KEY"),
            openai_api_version=os.environ.get("AZURE_OPENAI_API_VERSION")
        )
        qa = self.create_conversational_qa_chain(cl_llm)
        return qa



import os
from langchain.schema.embeddings import Embeddings
from langchain.vectorstores.pgvector import PGVector
from langchain.vectorstores.faiss import FAISS
from langchain.schema.vectorstore import VectorStore

from rag_doc_search.src.enums.provider import AIProvider, VectorStoreProvider
from rag_doc_search.src.enums.search_type import RetrieverSearchType
from rag_doc_search.utils.miscellaneous import get_logger


class Config:
    """
    A class representing configuration settings for the System which can be used to initialize the llm model, vector stores and its retrivers argument

    ```
    Note: Ensure that the necessary environment variables are set before initializing the configuration.
    If AI Provider is OPENAI then ENV OPENAI_API_KEY is required.
    If AI Provider is AZURE OPENAI then ENV AZURE_OPENAI_API_KEY, AZURE_OPENAI_ENDPOINT are required and AZURE_OPENAI_API_VERSION is optional, default is '2023-05-15'.
    If AI Provider is BEDROCK then ENV AWS_ACCESS_KEY,AWS_SECRET_ACCESS_KEY are required and AWS_REGION is optional, default is 'us-east-1'.
    If Vector Store Provider is PGVector then PGVECTOR_HOST, PGVECTOR_PORT,  PGVECTOR_DATABASE,  PGVECTOR_USER and PGVECTOR_PASSWORD are required
    """

    _instance = None

    def __new__(cls, config: dict = None):
        if cls._instance is None:
            if config is not None:
                cls._instance = super(Config, cls).__new__(cls)
                cls._instance._validate_and_initialize(config)
            else:
                raise ValueError(
                    "Config is not provided while creating singleton instance."
                )
        return cls._instance

    def _validate_and_initialize_ai_provider(self, config_json: dict):
        """
        Validates and initializes the AI provider based on the configuration JSON.
        Also validates the required environment variables for the language model (LLM) as per provider
        and also initializes AI LLM config properties.

        Parameters:
        - `config_json`: A dictionary containing configuration settings.

        Raises:
        - `ValueError`: If the configuration is invalid or missing required properties.
        """
        ai_provider_str = config_json.get("ai_provider", "")
        try:
            self.ai_provider = AIProvider(ai_provider_str)
        except ValueError:
            raise ValueError(
                f"Invalid value for 'ai_provider'. Expected values: {', '.join(member.value for member in AIProvider)}"
            )

        match self.ai_provider:
            case AIProvider.OPENAI:
                if not os.environ.get("OPENAI_API_KEY"):
                    raise ValueError(
                        "OPENAI_API_KEY environment variable is required for OpenAI"
                    )
            
            case AIProvider.AZURE_OPENAI:
                if not os.environ.get("AZURE_OPENAI_API_KEY"):
                    raise ValueError(
                        "AZURE_OPENAI_API_KEY environment variable is required for Azure OpenAI"
                    )
                if not os.environ.get("AZURE_OPENAI_ENDPOINT"):
                    raise ValueError(
                        "AZURE_OPENAI_ENDPOINT environment variable is required for Azure OpenAI"
                    )
                if not os.environ.get("AZURE_OPENAI_API_VERSION"):
                    os.environ["AZURE_OPENAI_API_VERSION"] = "2023-05-15"
                    print( os.environ.get("AZURE_OPENAI_API_VERSION") )
                    self.logger.warning(
                        'AZURE_OPENAI_API_VERSION key not found in enviroment variables, so "2023-05-15" will be used as default value.'
                    )
                    print(
                        'AZURE_OPENAI_API_VERSION key not found in enviroment variables, so "2023-05-15" will be used as default value.'
                    )

            case AIProvider.BEDROCK:
                if not os.environ.get("AWS_ACCESS_KEY") or not os.environ.get(
                    "AWS_SECRET_ACCESS_KEY"
                ):
                    raise ValueError(
                        "AWS_ACCESS_KEY and AWS_SECRET_ACCESS_KEY environment variables are required for Bedrock"
                    )

                if not os.environ.get("AWS_REGION"):
                    os.environ["AWS_REGION"] = "us-east-1"
                    print( os.environ.get("AWS_REGION") )
                    self.logger.warning(
                        'AWS_REGION key not found in enviroment variables, so "us-east-1" will be used as default value.'
                    )
                    print(
                        'AWS_REGION key not found in enviroment variables, so "us-east-1" will be used as default value.'
                    )
            case _:
                self.logger.warning("Default case AIProvider")

    def _validate_and_initialize_vector_store_provider(self, config_json: dict):
        """
        Validates and initializes the vector store provider based on the configuration JSON.
        Also validates and initializes vector store config properties and checks required environment variables as per provider.

        Parameters:
        - `config_json`: A dictionary containing configuration settings.

        Raises:
        - `ValueError`: If the configuration is invalid or missing required properties.
        """
        vector_store_provider_str = config_json.get("vector_store_provider", "")
        try:
            self.vector_store_provider = VectorStoreProvider(vector_store_provider_str)
        except ValueError:
            raise ValueError(
                f"Invalid value for 'vector_store_provider'. Expected values: {', '.join(member.value for member in VectorStoreProvider)}"
            )
        match self.vector_store_provider:
            case VectorStoreProvider.FAISS:
                self.faiss_vector_embeddings_location = config_json.get(
                    "faiss_vector_embeddings_location", ""
                )
                self.faiss_index_name = config_json.get("faiss_index_name", "index")
                if not self.faiss_vector_embeddings_location:
                    raise ValueError(
                        "faiss_vector_embeddings_location are required for FAISS"
                    )
                # if self.faiss_vector_embeddings_location.startswith("./"):
                #     self.faiss_vector_embeddings_location = (
                #         self.faiss_vector_embeddings_location.replace(
                #             "./", os.path.dirname(__file__) + "/"
                #         )
                #     )
                self.logger.info(f"{self.faiss_vector_embeddings_location}")
                if not (os.path.exists(self.faiss_vector_embeddings_location)):
                    raise ValueError(
                        "Invalid Directory path for FAISS vector embeddings"
                    )
            case VectorStoreProvider.PGVector:
                self.collection_name = config_json.get("collection_name", "")
                if not self.collection_name:
                    raise ValueError("collection_name is required for PGVECTOR")

                if (
                    not os.environ.get("PGVECTOR_HOST")
                    or not os.environ.get("PGVECTOR_PORT")
                    or not os.environ.get("PGVECTOR_DATABASE")
                    or not os.environ.get("PGVECTOR_USER")
                    or not os.environ.get("PGVECTOR_PASSWORD")
                ):
                    raise ValueError(
                        "PGVECTOR_HOST, PGVECTOR_PORT, PGVECTOR_DATABASE, PGVECTOR_USER and PGVECTOR_PASSWORD environment variables are required for PgVector"
                    )
            case _:
                self.logger.warning("Default case VectorStoreProvider")

    def _validate_and_initialize_retriever_arguments(self, config_json: dict):
        """
        Validates and initializes the retriever arguments based on the configuration JSON.
        Also validates and initializes retriever config properties.

        Parameters:
        - `config_json`: A dictionary containing configuration settings.

        Raises:
        - `ValueError`: If the configuration is invalid or missing required properties.
        """

        retriever_args = config_json.get("retriever", {})
        search_type_str = retriever_args.get("search_type", "")
        try:
            self.retriever_search_type = RetrieverSearchType(search_type_str)
        except ValueError:
            raise ValueError(
                f"Invalid value for 'search_type'. Expected values: {', '.join(member.value for member in RetrieverSearchType)}"
            )

        search_args = retriever_args.get("search_args", {})

        self.retriever_search_args_k = int(search_args.get("k", 4))
        self.retriever_search_args_fetch_k = int(search_args.get("fetch_k", 20))

        self.retriever_search_args_lambda_mult = float(
            config_json.get("lambda_mult", 0.5)
        )
        if not (0 <= self.retriever_search_args_lambda_mult <= 1):
            raise ValueError(
                "Invalid lambda_mult value in config -> retriever -> search args"
            )

        self.retriever_search_args_score_threshold = float(
            config_json.get("score_threshold", 0)
        )

    def _validate_and_initialize(self, config_json: dict):
        """
        Validates and initializes various configuration properties based on the provided JSON.

        Parameters:
        - `config_json`: A dictionary containing configuration settings.

        Raises:
        - `ValueError`: If the configuration is invalid or missing required properties.
        """
        self.logger = get_logger()
        self.logger.info("Vaidating the config")

        self._validate_and_initialize_ai_provider(config_json)
        self._validate_and_initialize_vector_store_provider(config_json)
        self._validate_and_initialize_retriever_arguments(config_json)

        # Validate Embeddings model, llm, llm_temperature, and llm_max_output_tokens
        self.embeddings_model = config_json.get("embeddings_model", "")
        if not self.embeddings_model:
            raise ValueError("No value provided for the key 'embeddings_model'")

        self.llm = config_json.get("llm", "")
        if not self.llm:
            raise ValueError("No value provided for the key 'llm'")

        self.llm_temperature = float(config_json.get("llm_temperature", 0.1))
        if not (0 <= self.llm_temperature <= 2):
            raise ValueError("Invalid llm_temperature value in config")

        self.llm_max_output_tokens = int(config_json.get("llm_max_output_tokens", 500))
        if self.llm_max_output_tokens <= 0:
            raise ValueError("Invalid llm_max_output_tokens value in config")

        # Set the name property
        self.name = config_json.get("name", "")

        self.vector_store = None

    def get_vector_store(self, embeddings: Embeddings) -> VectorStore:
        """
        Uses the vector store provider to create a requested provider using the required information for that vector store such as FAISS and PgVector.

        Parameters:
        - `embeddings`: An instance of the Embeddings class.

        Returns:
        A VectorStore instance based on the specified provider.

        Raises:
        - `ValueError`: If the specified vector store provider is not supported.
        """
        if self.vector_store is None:
            match self.vector_store_provider:
                case VectorStoreProvider.FAISS:
                    self.vector_store = FAISS.load_local(
                        folder_path=self.faiss_vector_embeddings_location,
                        embeddings=embeddings,
                        index_name=self.faiss_index_name,
                        allow_dangerous_deserialization=True,
                    )

                case VectorStoreProvider.PGVector:
                    CONNECTION_STRING = PGVector.connection_string_from_db_params(
                        driver="psycopg2",
                        host=os.environ.get("PGVECTOR_HOST", "localhost"),
                        port=int(os.environ.get("PGVECTOR_PORT", "5432")),
                        database=os.environ.get("PGVECTOR_DATABASE", "postgres"),
                        user=os.environ.get("PGVECTOR_USER", "postgres"),
                        password=os.environ.get("PGVECTOR_PASSWORD", "postgres"),
                    )
                    self.vector_store = PGVector(
                        collection_name="city_bot",
                        connection_string=CONNECTION_STRING,
                        embedding_function=self.embeddings,
                    )
                    self.vector_store.as_retriever
                case _:
                    self.logger.warning("Default case VectorStoreProvider")
        return self.vector_store

    def get_retriever_args(self) -> dict:
        """
        Retrieves retriever information from the configuration.

        Returns:
        A dictionary containing retriever configuration properties which can be used in vector store retriever.

        Raises:
        - `ValueError`: If the retriever configuration is invalid or missing required properties.
        """
        retriever_args = {
            "k": self.retriever_search_args_k,
            "fetch_k": self.retriever_search_args_fetch_k,
        }
        match self.vector_store_provider:
            case RetrieverSearchType.mmr:
                if self.retriever_search_args_lambda_mult <= 0:
                    retriever_args["lambda_mult"] = self.retriever_search_args_fetch_k
            case RetrieverSearchType.similarity_score_threshold:
                retriever_args[
                    "score_threshold"
                ] = self.retriever_search_args_score_threshold

        return {
            "search_type": self.retriever_search_type.value,
            "search_args": retriever_args,
        }



DEFAULT_PROMPT_TEMPLATE = """
{context}

Human: Do not answer any questions from your own knowledge. Use content from above context to answer to answer the question inside the <q></q> XML tags.

<q>{question}</q>

Do not include any XML tags in the answer. If the answer is not in the context say "Sorry, I don't know as the answer was not found in the context"

Assistant:"""

DEFAULT_CHAT_HISTORY_PROMPT = """{chat_history}

Answer only with the new question.


Human: How would you ask the question considering the previous conversation: {question}


Assistant: Question:"""



from pydantic import BaseModel, Field


class UserPrompt(BaseModel):
    """User Promp schema."""
    prompt: str= Field("", description="A string that should not be empty or contain only whitespace", min_length=1)
